<div style="
    position: fixed;
    top: 10px; left: 50%; transform: translateX(-50%);
    background-color: rgba(0, 0, 0, 0.5);
    color: white;
    padding: 2px 10px;
    border: 2px solid white;
    border-radius: 5px;
    z-index: 9999;
    font-size: 16px;
    text-align: center;
">
    {{ title }}
</div>